package org.planet.snap.deployer;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class MainOptionsWizardDialog extends TitleAreaDialog {
  private SnapDeployer win;
  private Button signButton;
  private Button listButton;
  private Button transformButton;
  private Button removeButton;

  public MainOptionsWizardDialog (Shell parentShell, SnapDeployer wd) {
    super (parentShell);
    win = wd;
  }

  protected Control createDialogArea (Composite parent) {
    Composite area = (Composite) super.createDialogArea (parent);
    Composite container = new Composite(area, SWT.NONE);
    container.setLayoutData (new GridData(GridData.FILL_BOTH));

    super.getShell().setText ("Snap Deployer");
    Image img = new Image (super.getShell().getDisplay(), "./images/snap_installer.jpg");
    super.getShell().setImage (img);

    setTitle ("SNAP Web Application Signer and Deployer Wizard");
    setMessage("Choose your type of execution.");
    setTitleImage (img);

    final Label label = new Label(container, SWT.NONE);
    label.setBounds (15, 27, 450, 18);
    label.setText ("Please choose what you want to do:");

    signButton = new Button(container, SWT.RADIO);
    signButton.setBounds(81, 65, 280, 30);
    signButton.setText("Sign and Deploy a new Snap Web Application");

    transformButton = new Button(container, SWT.RADIO);
    transformButton.setBounds(81, 92, 280, 30);
    transformButton.setText("Sign and Deploy a non-Snap Web Application");

    listButton = new Button(container, SWT.RADIO);
    listButton.setBounds(81, 121, 290, 25);
    listButton.setText("Show already deployed Web Applications");

    removeButton = new Button(container, SWT.RADIO);
    removeButton.setBounds(81, 144, 290, 30);
    removeButton.setText("Remove an already deployed Web Application");

    final Label label_1 = new Label(container, SWT.SEPARATOR | SWT.HORIZONTAL);
    label_1.setBounds (7, 200, 485, 5);

    return area;
  }

  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.NEXT_ID, IDialogConstants.NEXT_LABEL,
                 true);
    createButton(parent, IDialogConstants.CANCEL_ID,
                 IDialogConstants.CANCEL_LABEL, false);

    Button cancel = getButton (IDialogConstants.CANCEL_ID);
    cancel.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        close();
      }
    });

    Button next = getButton (IDialogConstants.NEXT_ID);
    next.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        boolean isSignButtonSelected = signButton.getSelection();
        boolean isTransformButtonSelected = transformButton.getSelection();
        boolean isListButtonSelected = listButton.getSelection();
        boolean isRemoveButtonSelected = removeButton.getSelection();
        Point p = getShell().getLocation();

        close();

        if (isSignButtonSelected) {
          win.openKeyGenDialog (p, true);
        }
        else if (isTransformButtonSelected) {
          win.openKeyGenDialog (p, false);
        }
        else if (isListButtonSelected) {
          win.openListAndRemoveDialog (p, false);
        }
        else if (isRemoveButtonSelected) {
          win.openListAndRemoveDialog (p, true);
        }
      }
    });
  }

  protected Point getInitialSize() {
    return new Point(500, 375);
  }
}

