package org.planet.snap.deployer;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;

import java.util.Properties;
import org.planet.snap.Context;

public class SignAndDeployWarFileDialog extends TitleAreaDialog {
  private SnapDeployer win;
  private ProgressBar progressBar;
  private boolean success = false;
  private String warFile;
  private String p2pUrl;
  private Button next;

  public SignAndDeployWarFileDialog (Shell parentShell, SnapDeployer wd, String war, String p2p) {
    super (parentShell);
    win = wd;
    warFile = war;
    p2pUrl = p2p;
  }

  protected Control createDialogArea (Composite parent) {
    Composite area = (Composite) super.createDialogArea(parent);
    Composite container = new Composite(area, SWT.NONE);
    container.setLayoutData(new GridData(GridData.FILL_BOTH));

    super.getShell().setText ("Snap Deployer");
    Image img = new Image (super.getShell().getDisplay(), "./images/snap_installer.jpg");
    super.getShell().setImage (img);

    setTitle ("SNAP Web Application Signer and Deployer Wizard");
    setMessage ("Signing and deploying web application into SNAP.");
    setTitleImage (img);

    final Label label = new Label(container, SWT.NONE);
    label.setBounds(15, 27, 450, 18);
    label.setText("Please press 'Next' to start signing and deploying web application...");

    progressBar = new ProgressBar(container, SWT.NONE);
    progressBar.setBounds(50, 151, 395, 10);
    progressBar.setMinimum (0);
    progressBar.setMaximum(100);

    success = true;
    Properties p = null;
    try {
      p = win.extractWarInfo (warFile);
    } catch (Exception e) {
      success = false;
    }

    final Label label_1 = new Label(container, SWT.NONE);
    label_1.setBounds(56, 61, 394, 20);
    if (success) {
      label_1.setText ("Name:\t\t " + p.getProperty (Context.SNAP_APPCONTEXT));
    }
    else {
      label_1.setText ("Name: Exception occured while reading war file.");
    }

    final Label label_2 = new Label (container, SWT.NONE);
    label_2.setBounds (56, 79, 394, 20);
    if (success) {
      label_2.setText ("Description:\t " + p.getProperty (Context.SNAP_APPNAME));
    }
    else {
      label_2.setText ("Description: Exception occured while reading war file.");
    }

    final Label label_3 = new Label(container, SWT.NONE);
    label_3.setBounds(56, 97, 394, 20);
    label_3.setText ("P2PURL:\t " + p2pUrl);

    final Label label_4 = new Label(container, SWT.SEPARATOR | SWT.HORIZONTAL);
    label_4.setBounds (7, 200, 485, 5);

    return area;
  }

  protected void createButtonsForButtonBar(Composite parent) {
    createButton (parent, IDialogConstants.BACK_ID, IDialogConstants.BACK_LABEL, false);

    createButton(parent, IDialogConstants.NEXT_ID,
                 IDialogConstants.NEXT_LABEL, false);
    createButton(parent, IDialogConstants.CANCEL_ID,
                 IDialogConstants.CANCEL_LABEL, false);

    Button cancel = getButton (IDialogConstants.CANCEL_ID);
    cancel.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        close();
      }
    });

    final Button back = getButton (IDialogConstants.BACK_ID);
    back.setEnabled (true);
    back.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        close();
        win.openWarDeploymentDialog (getShell().getLocation());
      }
    });

    next = getButton (IDialogConstants.NEXT_ID);
    if (!success) {
      next.setEnabled (false);
    }
    next.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        try {
          next.setEnabled (false);
          back.setEnabled (false);
          win.insertAndSignWar (warFile, p2pUrl, progressBar);
          success = true;
        } catch (Exception e) {
          e.printStackTrace();
          success = false;
        }

        if (success) {
          Point p = getShell().getLocation();
          close();
          win.openFinishDialog (p);
        }
        else {
          close();
          win.openErrorDialog();
        }
      }
    });
  }

  protected Point getInitialSize() {
    return new Point(500, 375);
  }
}
