package org.planet.snap.ds;

import java.sql.*;
import javax.sql.*;
import org.planet.snap.Application;
import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.dbcp.DelegatingConnection;
//import org.apache.commons.dbcp.PoolGuardConnectionWrapper;
import org.planet.snap.interception.DBInterceptor;

public class SnapDataSource extends BasicDataSource {
  private Application app;

  /**
   * DataSource creation method override
   * @return DataSource Returns a Snap DataSource
   * @throws SQLException If something goes wrong ;-)
   */
  protected synchronized DataSource createDataSource() throws SQLException {
    // Load the application's database (if not loaded)
    if (app == null) throw new SQLException ("Snap application reference is null!!!");
    try {
      this.setDriverClassName ("org.hsqldb.jdbcDriver");
      this.setUrl ("jdbc:hsqldb:hsql://localhost:" + app.dataSourceCreateDB());
    } catch (Exception e) {
      e.printStackTrace();
    }
    return super.createDataSource();
  }

  /**
   * Connection getter method override: a dynamic proxy instance is returned.
   * This proxy intercepts CUD statements and sends event messages accordingly to other
   * listening database replicas.
   * @return Connection Database connection
   * @throws SQLException If something goes wrong ;-)
   */
  public Connection getConnection() throws SQLException {
    return (Connection) DBInterceptor.newInstance (createDataSource().getConnection(), app);
  }

  /**
   * Setter method for linking this DataSource to an existent Snap application
   * @param a Application The Snap application to link to this DataSource
   */
  public void setApplication (Application a) {
    if (a != null) {
      app = a;
    }
  }
}
