package org.planet.snap.interception;

import org.planet.snap.Application;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.logging.Logger;
import java.util.*;

/**
 * Dynamic proxy for intercepting DB calls and monitor activity
 */
public class DBInterceptor implements InvocationHandler {
  // Proxy's object implementation
  private Object obj;
  private Application app;

  protected transient Logger logger = Logger.getLogger (this.getClass().getName());

  /**
   * This method returns a new instance of a Proxy
   * @param obj Object Object to be intercepted by the proxy
   * @param props Properties Object's initialization properties
   * @return Object Proxy instance
   */
  public static Object newInstance (Object obj, Application app) {
    ArrayList<Class> interfaces = new ArrayList<Class>();

    for (Class c : obj.getClass().getInterfaces()) {
        interfaces.add (c);
      }

    for (Class c : obj.getClass().getSuperclass().getInterfaces()) {
      interfaces.add (c);
    }

    Class[] interfs = new Class[interfaces.size()];
    Iterator<Class> it = interfaces.iterator();
    int idx = 0;
    while (it.hasNext()) {
      interfs[idx] = it.next();
      idx++;
    }

    return java.lang.reflect.Proxy.newProxyInstance (obj.getClass().getClassLoader(), interfs, new DBInterceptor (obj, app));
  }

  public DBInterceptor (Object o, Application a) {
    obj = o;
    app = a;
  }

  /**
   * This method is called by the Proxy before invoking the interface method
   * @param proxy Object Proxy class object
   * @param m Method Method to invoke
   * @param arguments Object[] Arguments packed in an object array
   * @throws Throwable
   * @return Object Invocation result
   */
  public Object invoke (Object proxy, Method m, Object[] arguments) throws Throwable {
    String method = m.getName();

    // Connection class
    if ((method.equals ("createStatement") || (method.equals ("prepareStatement") || (method.equals ("prepareCall")) && (obj instanceof java.sql.Connection)))) {
      logger.info ("Intercepted java.sql.Connection creation.");
      return ((java.sql.Statement) DBInterceptor.newInstance (m.invoke (obj, arguments), app));
    }

    // Statement class
    if ((method.equals ("execute") || method.equals ("executeQuery") || method.equals ("executeUpdate")) && (obj instanceof java.sql.Statement)) {
      logger.info ("Intercepted java.sql.Statement execution.");
      app.setLastDatabaseCall (System.currentTimeMillis());
    }

    return m.invoke (obj, arguments);
  }
}
